import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.List;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

public class CyclicModel extends JPanel
{
	class CyclicStart implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			start();
			repaint();
		}
	}

	class CyclicStop implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			stop();
			if (!_simultaneous)
				doReset();
			lighton = false;
			setLight();
			repaint();
		}
	}

	class CyclicReset implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			doReset();
		}
	}

	class CyclicSequential implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			_simultaneous = false;
			doReset();
		}
	}

	class CyclicProton implements ActionListener
	{
		boolean _protonsOn = false;
		
		
		public CyclicProton(boolean protonsOn)
		{
			_protonsOn = protonsOn;
		}


		public void actionPerformed(ActionEvent e)
		{
			protons = _protonsOn;
			repaint();
		}
	}

	class CyclicFilled implements ActionListener
	{
		boolean _filled = false;


		public CyclicFilled(boolean isfilled)
		{
			_filled = isfilled;
		}

		
		public void actionPerformed(ActionEvent e)
		{
			filled = _filled;
			repaint();
		}
	}
	
	class CyclicSimultaneous implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			setSimultaneous();
			if (_simulationOn == 0) {
				lighton = false; //sets light when Simultaneous button is selected
				setLight();
			}
			_simulationOn = 1; //sets snapshot mode
			repaint();
		}
	}


	private static final String POINTS_FILE = "src/points.txt";

// pre-load icons so that there's no delay when sequential turns on
	private static final ImageIcon ICON_ON = new ImageIcon(CyclicModel.class.getResource("src/cyclicon.jpg"));
	private static final ImageIcon ICON_OFF = new ImageIcon(CyclicModel.class.getResource("src/cyclicoff.jpg"));
	
	
	private static boolean protons = false;
	private static boolean lighton = false;
	private static boolean changelight = false;
	private static boolean filled = true;

	private final Point[] _points;
	private final JLabel _backgroundLabel;
	private int _simulationOn = 0; //0=off, 1=snapshot, 2=continuous
	private boolean _simultaneous;
		
	private int offset = 0;


	public CyclicModel()
	{
		lighton = false;
		_backgroundLabel = new JLabel();
		setLight();

		_backgroundLabel.setBorder(null);
		add(_backgroundLabel);
		setBorder(null);

		try {
			_points = parsePoints();
		}
		catch (IOException e) {
			throw new RuntimeException("Error reading points file: " + e.getMessage());
		}
		
		stop(); //Have the bitmap on the screen
	}


	public static void main(String args[])
	{
		JFrame f = new JFrame();
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		CyclicModel cyclic = new CyclicModel();
		//timer = new Timer(500, cyclic);
		f.setTitle("Cyclic Models for Simple Circuits");
		f.getContentPane().setLayout(new BorderLayout());
		f.setBackground(Color.gray);
		f.setSize(490,610);

		f.getContentPane().add(cyclic, BorderLayout.CENTER);
		JPanel p = new JPanel(new GridLayout(5,3));

		JButton startButton = new JButton("Start");
		startButton.addActionListener(cyclic.cyclicStartFactory());
		
		JButton stopButton = new JButton("Stop");
		stopButton.addActionListener(cyclic.cyclicStopFactory());
		
		JButton resetButton = new JButton("Reset");
		resetButton.addActionListener(cyclic.cyclicResetFactory());
		
		JButton quitButton = new JButton("Quit");
		quitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				System.exit(0);
			}
		});

		JRadioButton simultaneousButton = new JRadioButton("Simultaneous");
		simultaneousButton.addActionListener(cyclic.cyclicSimultaneousFactory());
	// final for reset button listener
		final JRadioButton sequentialButton = new JRadioButton("Sequential");
		sequentialButton.addActionListener(cyclic.cyclicSequentialFactory());
		sequentialButton.setSelected(true);
		resetButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				sequentialButton.setSelected(true);
			}
		});

		ButtonGroup group = new ButtonGroup();
		group.add(sequentialButton);
		group.add(simultaneousButton);
		

		JRadioButton protonYesButton = new JRadioButton("On");
		protonYesButton.addActionListener(cyclic.cyclicProtonFactory(true));
		
		JRadioButton protonNoButton = new JRadioButton("Off");
		protonNoButton.addActionListener(cyclic.cyclicProtonFactory(false));
		protonNoButton.setSelected(true);

		group = new ButtonGroup();
		group.add(protonYesButton);
		group.add(protonNoButton);

		JRadioButton filledYesButton = new JRadioButton("On");
		filledYesButton.addActionListener(cyclic.cyclicFilledFactory(true));
		JRadioButton filledNoButton = new JRadioButton("Off");
		filledNoButton.addActionListener(cyclic.cyclicFilledFactory(false));
		filledYesButton.setSelected(true);

		group = new ButtonGroup();
		group.add(filledYesButton);
		group.add(filledNoButton);

		p.add(new Label("Mode:"));
		p.add(sequentialButton);
		p.add(simultaneousButton);

		p.add(new Label("Protons:"));
		p.add(protonYesButton);
		p.add(protonNoButton);
		
		p.add(new Label("Filled:"));
		p.add(filledYesButton);
		p.add(filledNoButton);

		p.add(startButton);
		p.add(stopButton);
		p.add(resetButton);
		p.add(quitButton);
		
		f.getContentPane().add(p, BorderLayout.SOUTH);
		f.show();
		//start();
	}
	
	private static Point[] parsePoints()
	throws IOException
	{
		List points = new java.util.ArrayList();
		
		Reader in = new InputStreamReader(CyclicModel.class.getResourceAsStream(POINTS_FILE));
		
		StringBuffer buffer = new StringBuffer();
		Integer x = null;
		Integer y = null;
		
		for (int i = in.read(); i > -1; i = in.read()) {
			char c = (char) i;
			if (Character.isWhitespace(c))
				continue;
			
			switch (c) {
				case ',':
					x = Integer.valueOf(buffer.toString());
					buffer.delete(0, buffer.length());
					y = null;
					break;
				case ';':
					if (x == null)
						throw new IllegalStateException("X not set.");
					y = Integer.valueOf(buffer.toString());
					buffer.delete(0, buffer.length());
					points.add(new Point(x.intValue(), y.intValue()));
					x = null;
					y = null;
					break;
				default:
					buffer.append(c);
			}
		}
		
		if (buffer.length() > 0) {
			if (x == null)
				throw new IllegalStateException("Input improperly terminated.");
			y = Integer.valueOf(buffer.toString());
			points.add(new Point(x.intValue(), y.intValue()));
		}
		
		final double max = 2.5;
		
		Point last = null;
		for (int i = 0; i < points.size(); i++) {
			Point point = (Point) points.get(i);
			if (last != null) {
				double diff = Math.sqrt(Math.pow(point.getX() - last.getX(), 2) +
					Math.pow(point.getY() - last.getY(), 2));
				
				if (diff > max)
					System.out.println(Long.toString(Math.round(diff)) + " pixel difference between (" + (int) point.getX() + "," + (int) point.getY() + ") and (" + (int) last.getX() + "," + (int) last.getY() + ").");
			}
			last = point;
		}
		
		return (Point[]) points.toArray(new Point[points.size()]);
	}


	public CyclicStart cyclicStartFactory()
	{
		return new CyclicStart();
	}

	public CyclicStop cyclicStopFactory()
	{
		return new CyclicStop();
	}

	public CyclicReset cyclicResetFactory()
	{
		return new CyclicReset();
	}

	public CyclicProton cyclicProtonFactory(boolean protonsOn)
	{
		return new CyclicProton(protonsOn);
	}

	public CyclicFilled cyclicFilledFactory(boolean isFilled)
	{
		return new CyclicFilled(isFilled);
	}

	public CyclicSimultaneous cyclicSimultaneousFactory()
	{
		return new CyclicSimultaneous();
	}

	public CyclicSequential cyclicSequentialFactory()
	{
		return new CyclicSequential();
	}

	private void doReset()
	{
		stop();
		resetOffset();
		lighton = false;
		setLight();
		repaint();
	}

	public void paint(java.awt.Graphics g)
	{
		super.paint(g);

		if ((_simulationOn==1) || (_simulationOn==2)) {
			Point currentPoint;
			if (_simulationOn==2) {
				if (changelight) {
					setLight();
					changelight = false;
				}

				if (offset >= 444) { //This is the point where the lightbulb turns on
					lighton = true;
					changelight = true;
				}
			}
			
			setFont(new Font("Courier", Font.BOLD, 8));
			g.setColor(Color.red);

			int spacing = 10;
			int remaining = (int)(offset/spacing);
			for (int i = 0; i <= remaining; i++) {
				if ((offset - spacing*i) < _points.length) {
					if ((i%2 == 1) && (protons==true))  //If proton and protons=true
					{
						g.setColor(Color.red);
						currentPoint = _points[offset - spacing*i];
						g.drawString("+", currentPoint.x, currentPoint.y);
						if (filled) {
							g.fillOval(currentPoint.x - 2, currentPoint.y-7, 8,8);
						}
						else {
							g.drawOval(currentPoint.x - 2, currentPoint.y-7, 8,8);
						}
					}
					else //If electron or else
					{
						g.setColor(Color.black);
						currentPoint = _points[offset - spacing*i];
						g.drawString("-", currentPoint.x, currentPoint.y);
						if (filled) {
							g.fillOval(currentPoint.x - 2, currentPoint.y-7, 8,8);
						}
						else {
							g.drawOval(currentPoint.x - 2, currentPoint.y-7, 8,8);
						}
					}
					//currentPoint = points[offset - spacing*i];
					//g.fillOval(currentPoint.x-2, currentPoint.y-7, 11,11);
				}
			}
			
			if (_simulationOn == 1) {
//				_simulationOn = 0;
			}
			else {
				offset = offset+3; //spacing coefficient
			}
		}
	}
	
	public void resetOffset()
	{
		offset = 0;
		lighton = false;
		changelight = true;
	}

	private void setLight()
	{
		_backgroundLabel.setIcon(lighton ? ICON_ON : ICON_OFF);
	}
	
	private void setSimultaneous()
	{
		offset = 900;
		_simultaneous = true;
	}

	public void start()
	{
		_simulationOn = 2;
	}

	public void stop()
	{
		if ( _simultaneous )
			_simulationOn = 1;
		else
			_simulationOn = 0;
	}
}
